package mi.hdm.recipes;

import mi.hdm.exceptions.InvalidNutritionTableException;

import java.util.HashMap;
import java.util.Map;

public class NutritionTable {
    private final Map<Nutrition, Double> table;

    /**
     * Creates a new nutrition table that can be used in an ingredient. Expects values for all the nutrition values in order: Calories, Carbs, Fats, Proteins, Fibers, Salt.
     * All values supplied should be in relation to 100 grams or 1 piece.
     */
    public NutritionTable(double calories, double carbs, double fats, double proteins, double fibers, double salt) {
        this.table = new HashMap<>();
        if (calories < 0 || carbs < 0 || fats < 0 || proteins < 0 || fibers < 0 || salt < 0) {
            throw new InvalidNutritionTableException("Can't create NutritionTable where any of the arguments is <0");
        }

        table.put(Nutrition.CALORIES, calories);
        table.put(Nutrition.CARBS, carbs);
        table.put(Nutrition.FAT, fats);
        table.put(Nutrition.PROTEINS, proteins);
        table.put(Nutrition.FIBERS, fibers);
        table.put(Nutrition.SALT, salt);
    }

    private NutritionTable() {
        this.table = new HashMap<>();

        table.put(Nutrition.CALORIES, 0.0);
        table.put(Nutrition.CARBS, 0.0);
        table.put(Nutrition.FAT, 0.0);
        table.put(Nutrition.PROTEINS, 0.0);
        table.put(Nutrition.FIBERS, 0.0);
        table.put(Nutrition.SALT, 0.0);
    }

    public Map<Nutrition, Double> getTable() {
        return table;
    }

    public NutritionTable get() {
        return this;
    }

    //TODO: use big decimal instead of double to avoid this trouble?

    /**
     * Overridden equals method to account for rounding errors when calculating nutrition scores.
     *
     * @param o Object that is being compared
     * @return true if the given object is a NutritionTable and the difference between all of the values inside of the nutrition table are within a certain limit delta = 0.001
     */
    @Override
    public boolean equals(Object o) {
        if (o instanceof NutritionTable n) {
            double delta = 0.001;
            return n.getTable().get(Nutrition.CALORIES) - table.get(Nutrition.CALORIES) < delta
                    && n.getTable().get(Nutrition.CARBS) - table.get(Nutrition.CARBS) < delta
                    && n.getTable().get(Nutrition.FAT) - table.get(Nutrition.FAT) < delta
                    && n.getTable().get(Nutrition.PROTEINS) - table.get(Nutrition.PROTEINS) < delta
                    && n.getTable().get(Nutrition.FIBERS) - table.get(Nutrition.FIBERS) < delta
                    && n.getTable().get(Nutrition.SALT) - table.get(Nutrition.SALT) < delta;
        }
        return false;
    }

    public static NutritionTable empty() {
        return new NutritionTable();
    }
}
